/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997, 1998, 1999, 2000, 2001 The PHP Group             |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors:                                                             |
   | Florian Wesch fw@dividuum.de                                         |
   +----------------------------------------------------------------------+
 */

#include "php.h"
#include "php_ini.h"
#include "php_bf.h"
#include "bfinter.h"

/* True global resources - no need for thread safety here */
static int le_bf;

/* Every user visible function must have an entry in bf_functions[].
*/
function_entry bf_functions[] = {
	PHP_FE(executebf,	NULL)
	{NULL, NULL, NULL}	/* Must be the last line in bf_functions[] */
};

zend_module_entry bf_module_entry = {
	"bf",
	bf_functions,
	PHP_MINIT(bf),
	PHP_MSHUTDOWN(bf),
	NULL,	
	NULL,	
	PHP_MINFO(bf),
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_BF
ZEND_GET_MODULE(bf)
#endif


PHP_MINIT_FUNCTION(bf)
{
	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(bf)
{
	return SUCCESS;
}


PHP_MINFO_FUNCTION(bf)
{
    php_info_print_table_start();
    php_info_print_table_row(2, "Brainfuck Support", "enabled");
    php_info_print_table_row(2, "BFInter Version", BFINTERVERSION);
    php_info_print_table_end();
}

/* {{{ proto array executebf(string programm [, string input])
    */
PHP_FUNCTION(executebf)
{
	zval **programm, **input;
	int argc;
    
    int status;

    const int maxsize=16000;
    
    char *programminput=NULL;
    char programminputsize=0;
    
    unsigned char *output;
    int outputsize=0;
    
	argc = ZEND_NUM_ARGS();
	if (argc < 1 || argc > 2 || zend_get_parameters_ex(argc, &programm, &input) == FAILURE){
		WRONG_PARAM_COUNT;
	}

	switch (argc) {
		case 2:
			convert_to_string_ex(input);
			convert_to_string_ex(programm);

            programminput=Z_STRVAL_PP(input);
            programminputsize=Z_STRLEN_PP(input);
            break;
		case 1:
			convert_to_string_ex(programm);
            
            programminput=NULL;
            programminputsize=0;
			break;
	}

    
    output=(unsigned char*)emalloc(maxsize);
    
    status=executebf(Z_STRVAL_PP(programm), Z_STRLEN_PP(programm), 
                     programminput,         programminputsize, 
                     output,                &outputsize, 
                     maxsize,               4000,
					 BF_UNLIMITED);

    switch (status) {
        case BF_SUCCESS:
            output=(unsigned char*)erealloc(output,outputsize);
            RETVAL_STRINGL(output, outputsize, 0); 
            break;
        case BF_UNMATCHEDOPEN:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Unmatched [");
            RETURN_FALSE;
            break;
        case BF_UNMATCHEDCLOSE:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Unmatched ]");
            RETURN_FALSE;
            break;
        case BF_OUTPUTFULL:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Output too long (%d)",maxsize);
            RETURN_FALSE;
            break;
        case BF_INPUTEMPTY:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Not enough input");
            RETURN_FALSE;
            break;
        case BF_OVERFLOW:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Overflow");
            RETURN_FALSE;
            break;
        case BF_UNDERFLOW:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Underflow");
            RETURN_FALSE;
            break;
        case BF_UNKNOWNOPCODE:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Unknown Opcode");
            RETURN_FALSE;
            break;
        case BF_NOTSUPPORTED:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Feature not supported");
            RETURN_FALSE;
            break;
		case BF_TOOMANYINST:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Too many instructions");
            RETURN_FALSE;
            break;
		case BF_TOOMANYNESTEDLOOPS:
            efree(output);
            php_error(E_WARNING,"Brainfuck: Too many nested loops");
            RETURN_FALSE;
            break;
        default:
            efree(output);
            php_error(E_WARNING,"You are not supposed to be here"); // ;-)
            RETURN_FALSE;
            break;
    }
    
}
/* }}} */



/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 */
